package io.github.taha_cmd.api.http;

import io.github.taha_cmd.SignPathClientException;
import io.github.taha_cmd.SignPathClientSimpleLogger;
import okhttp3.Interceptor;
import okhttp3.Response;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.util.function.Function;

public class HttpClientRetryInterceptor implements Interceptor {

    public HttpClientRetryInterceptor(int numberOfRetries, SignPathClientSimpleLogger logger, Function<okhttp3.Response, String> errorMessageFormatter) {
        this.numberOfRetries = numberOfRetries;
        this.logger = logger;
        this.errorMessageFormatter = errorMessageFormatter;
    }

    private final int numberOfRetries;
    private final SignPathClientSimpleLogger logger;
    private final Function<Response, String> errorMessageFormatter;

    @NotNull
    @Override
    public Response intercept(@NotNull Chain chain) throws IOException {
        int numberOfTries = 0;
        int delayInSeconds = 1;
        okhttp3.Response lastResponse;
        Exception lastException = null;
        while (numberOfTries <= numberOfRetries) {

            if (numberOfTries > 0) {
                logger.log(String.format("Retry %s...",numberOfTries));
            }

            try {
                lastResponse = chain.proceed(chain.request());
                if (!isTransientHttpError(lastResponse.code())) {
                    // the response is either successful or not transient
                    // that's why there is no sense in trying again
                    // we return whatever we have
                    if (!lastResponse.isSuccessful()) {
                        throw new SignPathClientException(errorMessageFormatter.apply(lastResponse));
                    }

                    return lastResponse;
                }
            } catch (IOException ex) {
                lastException = ex;
                logger.log("encountered io exception -> retry triggered", ex);
            }
            try {
                Thread.sleep(delayInSeconds * 1000);
            } catch (InterruptedException ex) {
                logger.log(ex.getMessage(), ex);
            }

            numberOfTries++;
            delayInSeconds *= 3; // each try increases the delay three times
        }

        throw new IOException("API communication error.", lastException);
    }

    private static boolean isTransientHttpError(int httpResponseCode) {
        return httpResponseCode == 408 // Request Timeout
                || (httpResponseCode >= 500 && httpResponseCode < 600);
    }
}
